@@ -26,7 +26,6 @@ import java.util.List;
 import java.util.Map;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
-import java.util.regex.PatternSyntaxException;
 
 import javax.ws.rs.Path;
 import javax.ws.rs.core.MultivaluedMap;
@@ -41,8 +40,10 @@ public final class URITemplate {
     public static final String LIMITED_REGEX_SUFFIX = "(/.*)?";
     public static final String FINAL_MATCH_GROUP = "FINAL_MATCH_GROUP";
     private static final String DEFAULT_PATH_VARIABLE_REGEX = "([^/]+?)";
-    private static final String CHARACTERS_TO_ESCAPE = ".*";
-
+    private static final String CHARACTERS_TO_ESCAPE = ".*+";
+    private static final String SLASH = "/";
+    private static final String SLASH_QUOTE = "/;";
+    
     private final String template;
     private final List<String> variables = new ArrayList<String>();
     private final List<String> customVariables = new ArrayList<String>();
@@ -61,7 +62,8 @@ public final class URITemplate {
             UriChunk chunk = UriChunk.createUriChunk(templatePart);
             uriChunks.add(chunk);
             if (chunk instanceof Literal) {
-                String substr = escapeCharacters(chunk.getValue());
+                String encodedValue = HttpUtils.encodePartiallyEncoded(chunk.getValue(), false);
+                String substr = escapeCharacters(encodedValue);
                 literalChars.append(substr);
                 patternBuilder.append(substr);
             } else if (chunk instanceof Variable) {
@@ -141,7 +143,7 @@ public final class URITemplate {
         }
 
         Matcher m = templateRegexPattern.matcher(uri);
-        if (!m.matches()) {
+        if (!m.matches() || template.equals(SLASH) && uri.startsWith(SLASH_QUOTE)) {
             if (uri.contains(";")) {
                 // we might be trying to match one or few path segments
                 // containing matrix
@@ -150,14 +152,21 @@ public final class URITemplate {
                 List<PathSegment> uList = JAXRSUtils.getPathSegments(uri, false);
                 StringBuilder sb = new StringBuilder();
                 for (int i = 0; i < uList.size(); i++) {
-                    sb.append('/');
+                    String segment = null;
                     if (pList.size() > i && pList.get(i).getPath().indexOf('{') == -1) {
-                        sb.append(uList.get(i).getPath());
+                        segment = uList.get(i).getPath();
                     } else {
-                        sb.append(HttpUtils.fromPathSegment(uList.get(i)));
+                        segment = HttpUtils.fromPathSegment(uList.get(i));
                     }
+                    if (segment.length() > 0) {
+                        sb.append(SLASH);    
+                    }
+                    sb.append(segment);
                 }
                 uri = sb.toString();
+                if (uri.length() == 0) {
+                    uri = SLASH;
+                }
                 m = templateRegexPattern.matcher(uri);
                 if (!m.matches()) {
                     return false;
@@ -168,17 +177,27 @@ public final class URITemplate {
         }
 
         // Assign the matched template values to template variables
+        int groupCount = m.groupCount();
+        
         int i = 1;
         for (String name : variables) {
-            String value = m.group(i++);
-            templateVariableToValue.add(name, value);
+            while (i <= groupCount) {
+                String value = m.group(i++);
+                if (value == null || value.length() == 0 && i < groupCount) {
+                    continue;
+                }
+                templateVariableToValue.add(name, value);
+                break;
+            }
         }
         // The right hand side value, might be used to further resolve
         // sub-resources.
-        int groupCount = m.groupCount();
         
-        String finalGroup = m.group(groupCount);
-        templateVariableToValue.putSingle(FINAL_MATCH_GROUP, finalGroup == null ? "/" : finalGroup);
+        String finalGroup = i > groupCount ? SLASH : m.group(groupCount);
+        if (finalGroup == null) {
+            finalGroup = SLASH;
+        }
+        templateVariableToValue.putSingle(FINAL_MATCH_GROUP, finalGroup);
 
         return true;
     }
@@ -343,11 +362,11 @@ public final class URITemplate {
             if (uriChunk == null || "".equals(uriChunk)) {
                 throw new IllegalArgumentException("uriChunk is empty");
             }
-            try {
-                return new Variable(uriChunk);
-            } catch (IllegalArgumentException e) {
-                return new Literal(uriChunk);
+            UriChunk uriChunkRepresentation = Variable.create(uriChunk);
+            if (uriChunkRepresentation == null) {
+                uriChunkRepresentation = Literal.create(uriChunk);
             }
+            return uriChunkRepresentation;
         }
 
         public abstract String getValue();
@@ -361,11 +380,17 @@ public final class URITemplate {
     private static final class Literal extends UriChunk {
         private String value;
 
-        public Literal(String uriChunk) {
+        private Literal() {
+            // empty constructor
+        }
+
+        public static Literal create(String uriChunk) {
             if (uriChunk == null || "".equals(uriChunk)) {
                 throw new IllegalArgumentException("uriChunk is empty");
             }
-            value = uriChunk;
+            Literal literal = new Literal();
+            literal.value = uriChunk;
+            return literal;
         }
 
         @Override
@@ -379,33 +404,38 @@ public final class URITemplate {
      * Variable of URITemplate. Variable has either "{varname:pattern}" syntax or "{varname}".
      */
     private static final class Variable extends UriChunk {
-        private static final Pattern VARIABLE_PATTERN = Pattern.compile("(\\w[-\\w\\.]*)(\\:(.+))?");
+        private static final Pattern VARIABLE_PATTERN = Pattern.compile("(\\w[-\\w\\.]*[ ]*)(\\:(.+))?");
         private String name;
         private Pattern pattern;
 
+        private Variable() {
+            // empty constructor
+        }
+
         /**
          * Creates variable from stringified part of URI.
-         * 
-         * @param uriChunk chunk that depicts variable
-         * @throws IllegalArgumentException when param is null, empty or does not have variable syntax.
-         * @throws PatternSyntaxException when pattern of variable has wrong syntax.
+         *
+         * @param uriChunk uriChunk chunk that depicts variable
+         * @return Variable if variable was successfully created; null if uriChunk was not a variable
          */
-        public Variable(String uriChunk) throws IllegalArgumentException, PatternSyntaxException {
+        public static Variable create(String uriChunk) {
+            Variable newVariable = new Variable();
             if (uriChunk == null || "".equals(uriChunk)) {
-                throw new IllegalArgumentException("uriChunk is empty");
+                return null;
             }
             if (CurlyBraceTokenizer.insideBraces(uriChunk)) {
-                uriChunk = CurlyBraceTokenizer.stripBraces(uriChunk);
+                uriChunk = CurlyBraceTokenizer.stripBraces(uriChunk).trim();
                 Matcher matcher = VARIABLE_PATTERN.matcher(uriChunk);
                 if (matcher.matches()) {
-                    name = matcher.group(1).trim();
+                    newVariable.name = matcher.group(1).trim();
                     if (matcher.group(2) != null && matcher.group(3) != null) {
-                        pattern = Pattern.compile(matcher.group(3).trim());
+                        String patternExpression = matcher.group(3).trim();
+                        newVariable.pattern = Pattern.compile(patternExpression);
                     }
-                    return;
+                    return newVariable;
                 }
             }
-            throw new IllegalArgumentException("not a variable syntax");
+            return null;
         }
 
         public String getName() {
@@ -455,7 +485,7 @@ public final class URITemplate {
      * supported by Java regexp version.
      * 
      * @author amichalec
-     * @version $Rev: 19480 $
+     * @version $Rev$
      */
     static class CurlyBraceTokenizer {
 
