@@ -58,7 +58,6 @@ import org.apache.cxf.jaxrs.utils.HttpUtils;
 import org.apache.cxf.jaxrs.utils.InjectionUtils;
 import org.apache.cxf.jaxrs.utils.JAXBUtils;
 import org.apache.cxf.jaxrs.utils.schemas.SchemaHandler;
-import org.apache.cxf.staxutils.DepthExceededStaxException;
 import org.codehaus.jettison.mapped.Configuration;
 import org.codehaus.jettison.mapped.SimpleConverter;
 import org.codehaus.jettison.mapped.TypeConverter;
@@ -71,10 +70,6 @@ public class JSONProvider extends AbstractJAXBProvider  {
     private static final String MAPPED_CONVENTION = "mapped";
     private static final String BADGER_FISH_CONVENTION = "badgerfish";
     
-    static {
-        new SimpleConverter();
-    }
-    
     private ConcurrentHashMap<String, String> namespaceMap = 
         new ConcurrentHashMap<String, String>();
     private boolean serializeAsArray;
@@ -90,12 +85,6 @@ public class JSONProvider extends AbstractJAXBProvider  {
     private boolean ignoreNamespaces;
     private String convention = MAPPED_CONVENTION;
     private TypeConverter typeConverter;
-    private boolean attributesToElements;
-    
-    @Override
-    public void setAttributesToElements(boolean value) {
-        this.attributesToElements = value;
-    }
     
     public void setConvention(String value) {
         if (!MAPPED_CONVENTION.equals(value) && !BADGER_FISH_CONVENTION.equals(value)) {
@@ -185,8 +174,8 @@ public class JSONProvider extends AbstractJAXBProvider  {
         
         try {
             boolean isCollection = InjectionUtils.isSupportedCollectionOrArray(type);
-            Class<?> theGenericType = isCollection ? InjectionUtils.getActualType(genericType) : type;
-            Class<?> theType = getActualType(theGenericType, genericType, anns);
+            Class<?> theType = isCollection ? InjectionUtils.getActualType(genericType) : type;
+            theType = getActualType(theType, genericType, anns);
             
             Unmarshaller unmarshaller = createUnmarshaller(theType, genericType, isCollection);
             
@@ -206,16 +195,14 @@ public class JSONProvider extends AbstractJAXBProvider  {
             }
             if (isCollection) {
                 response = ((CollectionWrapper)response).getCollectionOrArray(theType, type, 
-                               org.apache.cxf.jaxrs.utils.JAXBUtils.getAdapter(theGenericType, anns)); 
+                                                         getAdapter(theType, anns)); 
             } else {
                 response = checkAdapter(response, type, anns, false);
             }
             return response;
             
         } catch (JAXBException e) {
-            handleJAXBException(e, true);
-        } catch (DepthExceededStaxException e) {
-            throw new WebApplicationException(413);
+            handleJAXBException(e);
         } catch (XMLStreamException e) {
             throw new WebApplicationException(e);
         } catch (WebApplicationException e) {
@@ -235,15 +222,12 @@ public class JSONProvider extends AbstractJAXBProvider  {
     
     protected XMLStreamReader createReader(Class<?> type, InputStream is) 
         throws Exception {
-        XMLStreamReader reader = null;
         if (BADGER_FISH_CONVENTION.equals(convention)) {
-            reader = JSONUtils.createBadgerFishReader(is);
+            return JSONUtils.createBadgerFishReader(is);
         } else {
-            reader = JSONUtils.createStreamReader(is, readXsiType, namespaceMap, getDepthProperties());
+            XMLStreamReader reader = JSONUtils.createStreamReader(is, readXsiType, namespaceMap);
+            return createTransformReaderIfNeeded(reader, is);
         }
-        reader = createTransformReaderIfNeeded(reader, is);
-        
-        return reader;
     }
     
     protected InputStream getInputStream(Class<Object> cls, Type type, InputStream is) throws Exception {
@@ -317,7 +301,7 @@ public class JSONProvider extends AbstractJAXBProvider  {
             }
             
         } catch (JAXBException e) {
-            handleJAXBException(e, false);
+            handleJAXBException(e);
         } catch (XMLStreamException e) {
             throw new WebApplicationException(e);
         } catch (Exception e) {
@@ -351,18 +335,11 @@ public class JSONProvider extends AbstractJAXBProvider  {
             } else {
                 qname = getCollectionWrapperQName(actualClass, genericType, firstObj, false);
             }
-            String prefix = "";
-            if (!ignoreNamespaces) {
-                if (namespaceMap.containsKey(qname.getNamespaceURI())) {
-                    prefix = namespaceMap.get(qname.getNamespaceURI());
-                    if (prefix.length() > 0) {
-                        prefix += ".";
-                    }
-                } else if (qname.getNamespaceURI().length() > 0) {
-                    prefix = "ns1.";
-                }
+            if (qname.getNamespaceURI().length() > 0) {
+                startTag = "{\"ns1." + qname.getLocalPart() + "\":[";
+            } else {
+                startTag = "{\"" + qname.getLocalPart() + "\":[";
             }
-            startTag = "{\"" + prefix + qname.getLocalPart() + "\":[";
             endTag = "]}";
         } else if (serializeAsArray) {
             startTag = "[";
@@ -374,8 +351,7 @@ public class JSONProvider extends AbstractJAXBProvider  {
         
         os.write(startTag.getBytes());
         if (firstObj != null) {
-            XmlJavaTypeAdapter adapter = 
-                org.apache.cxf.jaxrs.utils.JAXBUtils.getAdapter(firstObj.getClass(), anns);
+            XmlJavaTypeAdapter adapter = getAdapter(firstObj.getClass(), anns);
             marshalCollectionMember(JAXBUtils.useAdapter(firstObj, adapter, true),
                                     actualClass, genericType, encoding, os);
             while (it.hasNext()) {
@@ -415,19 +391,14 @@ public class JSONProvider extends AbstractJAXBProvider  {
     protected XMLStreamWriter createWriter(Object actualObject, Class<?> actualClass, 
         Type genericType, String enc, OutputStream os, boolean isCollection) throws Exception {
         
-        QName qname = getQName(actualClass, genericType, actualObject, true);
-        if (ignoreNamespaces && (isCollection  || dropRootElement)) {        
-            qname = new QName(qname.getLocalPart());
-        }
         if (BADGER_FISH_CONVENTION.equals(convention)) {
             return JSONUtils.createBadgerFishWriter(os);
         }
         
+        QName qname = getQName(actualClass, genericType, actualObject, true);
+        
         Configuration config = 
-            JSONUtils.createConfiguration(namespaceMap, 
-                                          writeXsiType && !ignoreNamespaces,
-                                          attributesToElements,
-                                          typeConverter);
+            JSONUtils.createConfiguration(namespaceMap, writeXsiType && !ignoreNamespaces, typeConverter);
         
         XMLStreamWriter writer = JSONUtils.createStreamWriter(os, qname, 
              writeXsiType && !ignoreNamespaces, config, serializeAsArray, arrayKeys,
