@@ -71,6 +71,7 @@ import org.apache.cxf.common.i18n.BundleUtils;
 import org.apache.cxf.common.logging.LogUtils;
 import org.apache.cxf.common.util.PackageUtils;
 import org.apache.cxf.common.util.StringUtils;
+import org.apache.cxf.helpers.XMLUtils;
 import org.apache.cxf.jaxrs.ext.MessageContext;
 import org.apache.cxf.jaxrs.ext.MessageContextImpl;
 import org.apache.cxf.jaxrs.ext.ProtocolHeaders;
@@ -112,7 +113,6 @@ public final class JAXRSUtils {
     private static final Logger LOG = LogUtils.getL7dLogger(JAXRSUtils.class);
     private static final ResourceBundle BUNDLE = BundleUtils.getBundle(JAXRSUtils.class);
     private static final String PROPAGATE_EXCEPTION = "org.apache.cxf.propagate.exception";
-    private static final String REPORT_FAULT_MESSAGE_PROPERTY = "org.apache.cxf.jaxrs.report-fault-message";
     
     private JAXRSUtils() {        
     }
@@ -303,14 +303,9 @@ public final class JAXRSUtils {
             new TreeMap<OperationResourceInfo, MultivaluedMap<String, String>>(
                 new OperationResourceInfoComparator(message, httpMethod));
 
-        MediaType requestType;
-        try {
-            requestType = requestContentType == null
+        MediaType requestType = requestContentType == null 
                                 ? ALL_TYPES : MediaType.valueOf(requestContentType);
-        } catch (IllegalArgumentException ex) {
-            throw new WebApplicationException(ex, 415);
-        }
-
+        
         int pathMatched = 0;
         int methodMatched = 0;
         int consumeMatched = 0;
@@ -398,7 +393,6 @@ public final class JAXRSUtils {
         org.apache.cxf.common.i18n.Message errorMsg = 
             new org.apache.cxf.common.i18n.Message(name, 
                                                    BUNDLE,
-                                                   message.get(Message.REQUEST_URI),
                                                    path,
                                                    httpMethod,
                                                    requestType.toString(),
@@ -406,17 +400,11 @@ public final class JAXRSUtils {
         if (!"OPTIONS".equalsIgnoreCase(httpMethod) && logNow) {
             LOG.warning(errorMsg.toString());
         }
-        Response response = 
-            createResponse(resource, message, errorMsg.toString(), status, methodMatched == 0);
-        throw new WebApplicationException(response);
+        ResponseBuilder rb = createResponseBuilder(resource, status, methodMatched == 0);
+        throw new WebApplicationException(rb.build());
         
     }    
     
-    public static boolean noResourceMethodForOptions(Response exResponse, String httpMethod) {
-        return exResponse != null && exResponse.getStatus() == 405 
-            && "OPTIONS".equalsIgnoreCase(httpMethod);
-    }
-    
     private static void logNoMatchMessage(OperationResourceInfo ori, 
         String path, String httpMethod, MediaType requestType, List<MediaType> acceptContentTypes) {
         if (!LOG.isLoggable(Level.FINE)) {
@@ -437,8 +425,7 @@ public final class JAXRSUtils {
         LOG.fine(errorMsg.toString());
     }
 
-    public static Response createResponse(ClassResourceInfo cri, Message msg,
-                                          String responseMessage, int status, boolean addAllow) {
+    public static ResponseBuilder createResponseBuilder(ClassResourceInfo cri, int status, boolean addAllow) {
         ResponseBuilder rb = Response.status(status);
         if (addAllow) {
             Set<String> allowedMethods = cri.getAllowedMethods();
@@ -453,10 +440,7 @@ public final class JAXRSUtils {
                 rb.header("Allow", "HEAD");
             }
         }
-        if (msg != null && MessageUtils.isTrue(msg.getContextualProperty(REPORT_FAULT_MESSAGE_PROPERTY))) {
-            rb.type(MediaType.TEXT_PLAIN_TYPE).entity(responseMessage);
-        }
-        return rb.build();
+        return rb;
     }
     
     private static boolean matchHttpMethod(String expectedMethod, String httpMethod) {
@@ -731,10 +715,9 @@ public final class JAXRSUtils {
             m.put(FormUtils.FORM_PARAM_MAP, params);
         
             if (mt == null || mt.isCompatible(MediaType.APPLICATION_FORM_URLENCODED_TYPE)) {
-                String enc = HttpUtils.getEncoding(mt, "UTF-8");
-                String body = FormUtils.readBody(m.getContent(InputStream.class), enc);
+                String body = FormUtils.readBody(m.getContent(InputStream.class), mt);
                 HttpServletRequest request = (HttpServletRequest)m.get(AbstractHTTPDestination.HTTP_REQUEST);
-                FormUtils.populateMapFromString(params, (String)body, enc, decode, request);
+                FormUtils.populateMapFromString(params, (String)body, decode, request);
             } else {
                 if (mt != null && "multipart".equalsIgnoreCase(mt.getType()) 
                     && MediaType.MULTIPART_FORM_DATA_TYPE.isCompatible(mt)) {
@@ -1255,21 +1238,7 @@ public final class JAXRSUtils {
     }
 
     public static QName convertStringToQName(String name) {
-        return convertStringToQName(name, "");
+        return XMLUtils.convertStringToQName(name, "");
     }
     
-    public static QName convertStringToQName(String name, String prefix) {
-        int ind1 = name.indexOf('{');
-        if (ind1 != 0) {
-            return new QName(name);
-        }
-        
-        int ind2 = name.indexOf('}');
-        if (ind2 <= ind1 + 1 || ind2 >= name.length() - 1) {
-            return null;
-        }
-        String ns = name.substring(ind1 + 1, ind2);
-        String localName = name.substring(ind2 + 1);
-        return new QName(ns, localName, prefix);
-    }
 }
