@@ -35,9 +35,7 @@ import java.util.Map;
 import java.util.Set;
 import java.util.TreeSet;
 
-import javax.ws.rs.WebApplicationException; 
 import javax.ws.rs.core.MediaType;
-import javax.ws.rs.core.Response;
 import javax.ws.rs.ext.MessageBodyReader;
 import javax.xml.bind.JAXBContext;
 import javax.xml.bind.JAXBElement;
@@ -69,21 +67,6 @@ import org.junit.Test;
 
 public class JSONProviderTest extends Assert {
 
-    @Test
-    @SuppressWarnings("unchecked")
-    public void testReadMalformedJson() throws Exception {
-        MessageBodyReader<?> p = new JSONProvider();
-        byte[] bytes = "junk".getBytes();
-         
-        try {
-            p.readFrom((Class)Tags.class, null, null, 
-                                           null, null, new ByteArrayInputStream(bytes));
-            fail("404 is expected");
-        } catch (WebApplicationException ex) {
-            assertEquals(Response.Status.BAD_REQUEST.getStatusCode(), ex.getResponse().getStatus());
-        }
-    }
-
     @SuppressWarnings("unchecked")
     @Test
     public void testReadEmbeddedArray() throws Exception {
@@ -283,22 +266,6 @@ public class JSONProviderTest extends Assert {
         
     }
     
-    @Test
-    public void testWriteToSingleTag2NoNs() throws Exception {
-        JSONProvider p = new JSONProvider();
-        p.setIgnoreNamespaces(true);
-        TagVO2 tag = createTag2("a", "b");
-        
-        ByteArrayOutputStream os = new ByteArrayOutputStream();
-        
-        p.writeTo(tag, TagVO2.class, TagVO2.class, TagVO2.class.getAnnotations(), 
-                  MediaType.APPLICATION_JSON_TYPE, new MetadataMap<String, Object>(), os);
-        
-        String s = os.toString();
-        assertEquals("{\"thetag\":{\"group\":\"b\",\"name\":\"a\"}}", s);
-        
-    }
-    
     @Test
     public void testWriteBookWithStringConverter() throws Exception {
         JSONProvider p = new JSONProvider();
@@ -368,7 +335,6 @@ public class JSONProviderTest extends Assert {
     public void testDropRootElement() throws Exception {
         JSONProvider p = new JSONProvider();
         p.setDropRootElement(true);
-        p.setIgnoreNamespaces(true);
         Map<String, String> namespaceMap = new HashMap<String, String>();
         namespaceMap.put("http://tags", "ns1");
         p.setNamespaceMap(namespaceMap);
@@ -388,32 +354,24 @@ public class JSONProviderTest extends Assert {
     public void testWriteQualifiedCollection() throws Exception {
         String data = "{\"ns1.tag\":[{\"group\":\"b\",\"name\":\"a\"}"
             + ",{\"group\":\"d\",\"name\":\"c\"}]}";
-        doWriteQualifiedCollection(false, false, false, data);
-    }
-    
-    @Test
-    public void testWriteQualifiedCollectionDropNs() throws Exception {
-        String data = "{\"tag\":[{\"group\":\"b\",\"name\":\"a\"}"
-            + ",{\"group\":\"d\",\"name\":\"c\"}]}";
-        doWriteQualifiedCollection(false, false, true, data);
+        doWriteQualifiedCollection(false, false, data);
     }
     
     @Test
     public void testWriteQualifiedCollection2() throws Exception {
         String data = "{{\"group\":\"b\",\"name\":\"a\"}"
             + ",{\"group\":\"d\",\"name\":\"c\"}}";
-        doWriteQualifiedCollection(true, false, false, data);
+        doWriteQualifiedCollection(true, false, data);
     }
     
     @Test
     public void testWriteQualifiedCollection3() throws Exception {
         String data = "[{\"group\":\"b\",\"name\":\"a\"}"
             + ",{\"group\":\"d\",\"name\":\"c\"}]";
-        doWriteQualifiedCollection(true, true, false, data);
+        doWriteQualifiedCollection(true, true, data);
     }
     
-    public void doWriteQualifiedCollection(boolean drop, boolean serializeAsArray, 
-                                           boolean ignoreNamespaces, String data) 
+    public void doWriteQualifiedCollection(boolean drop, boolean serializeAsArray, String data) 
         throws Exception {
         JSONProvider p = new JSONProvider();
         p.setCollectionWrapperName("{http://tags}tag");
@@ -422,8 +380,6 @@ public class JSONProviderTest extends Assert {
         Map<String, String> namespaceMap = new HashMap<String, String>();
         namespaceMap.put("http://tags", "ns1");
         p.setNamespaceMap(namespaceMap);
-        p.setIgnoreNamespaces(ignoreNamespaces);
-        
         List<TagVO2> tags = new ArrayList<TagVO2>();
         tags.add(createTag2("a", "b"));
         tags.add(createTag2("c", "d"));
@@ -500,6 +456,7 @@ public class JSONProviderTest extends Assert {
     }
     
     @Test
+    @Ignore("Enable once http://jira.codehaus.org/browse/JETTISON-104 gets resolved")
     public void testReadQualifiedArray() throws Exception {
         String data = "{\"ns1.thetag\":[{\"group\":\"b\",\"name\":\"a\"}"
             + ",{\"group\":\"d\",\"name\":\"c\"}]}";
@@ -556,7 +513,6 @@ public class JSONProviderTest extends Assert {
                   MediaType.APPLICATION_JSON_TYPE, new MetadataMap<String, Object>(), os);
         
         String s = os.toString();
-        System.out.println(s);
         assertEquals("{\"ns1.thetag\":{\"group\":\"b\",\"name\":\"a\"}}", s);
         
     }
@@ -574,8 +530,9 @@ public class JSONProviderTest extends Assert {
     }
     
     @Test
+    @Ignore("This is hitting http://jira.codehaus.org/browse/JETTISON-44")
     public void testWriteMixedContent() throws Exception {
-        doTestMixedContent("{\"Book\":{\"name\":\"CXF\",\"id\":125}}",
+        doTestMixedContent("{\"Book\":{\"name\":\"CXF\",\"id\":125,\"$\":\"\\n     \\n\"}}",
                            false, "book.xml");
     }
     
