@@ -23,7 +23,11 @@ package org.apache.cxf.systest.jaxrs;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
+import java.lang.reflect.InvocationHandler;
+import java.lang.reflect.Method;
+import java.net.URI;
 import java.net.URL;
+import java.util.ArrayList;
 import java.util.Calendar;
 import java.util.GregorianCalendar;
 import java.util.HashMap;
@@ -41,6 +45,7 @@ import javax.ws.rs.GET;
 import javax.ws.rs.HEAD;
 import javax.ws.rs.HeaderParam;
 import javax.ws.rs.MatrixParam;
+import javax.ws.rs.OPTIONS;
 import javax.ws.rs.POST;
 import javax.ws.rs.PUT;
 import javax.ws.rs.Path;
@@ -50,6 +55,7 @@ import javax.ws.rs.QueryParam;
 import javax.ws.rs.WebApplicationException;
 import javax.ws.rs.core.Context;
 import javax.ws.rs.core.Cookie;
+import javax.ws.rs.core.EntityTag;
 import javax.ws.rs.core.GenericEntity;
 import javax.ws.rs.core.HttpHeaders;
 import javax.ws.rs.core.MediaType;
@@ -58,17 +64,23 @@ import javax.ws.rs.core.PathSegment;
 import javax.ws.rs.core.Response;
 import javax.ws.rs.core.SecurityContext;
 import javax.ws.rs.core.StreamingOutput;
+import javax.ws.rs.core.UriBuilder;
 import javax.xml.bind.JAXBElement;
 import javax.xml.bind.annotation.adapters.XmlAdapter;
 import javax.xml.bind.annotation.adapters.XmlJavaTypeAdapter;
 import javax.xml.namespace.QName;
 import javax.xml.transform.dom.DOMSource;
 
+import org.apache.cxf.annotations.GZIP;
+import org.apache.cxf.common.util.ProxyHelper;
 import org.apache.cxf.helpers.XMLUtils;
 import org.apache.cxf.jaxrs.ext.Oneway;
+import org.apache.cxf.jaxrs.ext.search.SearchCondition;
+import org.apache.cxf.jaxrs.ext.search.SearchContext;
 import org.apache.cxf.phase.PhaseInterceptorChain;
 
 @Path("/bookstore")
+@GZIP(threshold = 1)
 public class BookStore {
 
     private Map<Long, Book> books = new HashMap<Long, Book>();
@@ -98,6 +110,15 @@ public class BookStore {
         System.out.println("PreDestroy called");
     }
     
+    @GET
+    @Path("/books/colon/{a}:{b}:{c}")
+    @Produces("application/xml")
+    public Book getBookWithColonMarks(@PathParam("a") String id1,
+                                      @PathParam("b") String id2,
+                                      @PathParam("c") String id3) throws BookNotFoundFault {
+        return doGetBook(id1 + id2 + id3);
+    }
+    
     @POST
     @Path("emptypost")
     public void emptypost() {
@@ -113,20 +134,66 @@ public class BookStore {
     }
     
     @GET
-    @Path("propogateexception")
+    @Path("webappexceptionXML")
+    public Book throwExceptionXML() {
+        
+        Response response = Response.status(406).type("application/xml")
+                            .entity("<Book><name>Exception</name><id>999</id></Book>")
+                            .build();
+        throw new WebApplicationException(response);
+    }
+    
+    @GET
+    @Path("tempredirect")
+    public Response tempRedirectAndSetCookies() {
+        URI uri = UriBuilder.fromPath("whatever/redirection")
+            .queryParam("css1", "http://bar").build();
+        return Response.temporaryRedirect(uri)
+                       .header("Set-Cookie", "a=b").header("Set-Cookie", "c=d")
+                       .build();
+    }
+    
+    @GET
+    @Path("setcookies")
+    public Response setComplexCookies() {
+        return Response.ok().header("Set-Cookie", 
+                                    "bar.com.anoncart=107894933471602436; Domain=.bar.com;"
+                                    + " Expires=Thu, 01-Oct-2020 23:44:22 GMT; Path=/")
+                                    .build();
+    }
+    
+
+    @GET
+    @Path("setmanycookies")
+    public Response setTwoCookies() {
+        return Response.ok().header("Set-Cookie", "JSESSIONID=0475F7F30A26E5B0C15D69; Path=/")
+            .header("Set-Cookie", "COOKIETWO=dummy; Expires=Sat, 20-Nov-2010 19:11:32 GMT; Path=/")
+            .header("Set-Cookie", "COOKIETWO=dummy2; expires=Sat, 20-Nov-2010 19:11:32 GMT; Path=/")
+            .build();
+    }
+    
+    @GET
+    @Path("propagate-exception")
     public Book propogateException() throws BookNotFoundFault {
         throw new BookNotFoundFault("Book Exception");
     }
     
     @GET
-    @Path("propogateexception2")
+    @Path("name-in-query")
+    @Produces("application/xml")
+    public Book getBookFromQuery(@QueryParam("name") String name) {
+        return new Book(name, 321L);
+    }
+    
+    @GET
+    @Path("propagate-exception2")
     public Book propogateException2() throws BookNotFoundFault {
-        PhaseInterceptorChain.getCurrentMessage().put("org.apache.cxf.propogate.exception", Boolean.FALSE);
+        PhaseInterceptorChain.getCurrentMessage().put("org.apache.cxf.propagate.exception", Boolean.FALSE);
         throw new BookNotFoundFault("Book Exception");
     }
     
     @GET
-    @Path("propogateexception3")
+    @Path("propagate-exception3")
     public Book propogateException3() throws BookNotFoundFault {
         PhaseInterceptorChain.getCurrentMessage().getExchange()
             .put("org.apache.cxf.systest.for-out-fault-interceptor", Boolean.TRUE);
@@ -182,10 +249,26 @@ public class BookStore {
         return doGetBook(url2.substring(index + 1));
     }
     
+    @OPTIONS
+    @Path("/options")
+    public Response getOptions() throws Exception {
+        return Response.ok().header("Allow", "POST")
+                            .header("Allow", "PUT")
+                            .header("Allow", "GET")
+                            .header("Allow", "DELETE")
+                            .build();
+    }
+    
+    @POST
+    @Path("post401")
+    public Response get401WithText() throws Exception {
+        return Response.status(401).entity("This is 401").build();
+    }
+    
     @POST
     @Path("/collections")
-    @Produces("application/xml")
-    @Consumes("application/xml")
+    @Produces({"application/xml", "application/json" })
+    @Consumes({"application/xml", "application/json" })
     public List<Book> getBookCollection(List<Book> bs) throws Exception {
         if (bs == null || bs.size() != 2) {
             throw new RuntimeException();
@@ -193,6 +276,24 @@ public class BookStore {
         return bs;
     }
     
+    @POST
+    @Path("/collections2")
+    @Produces({"application/xml", "application/json" })
+    @Consumes({"application/xml", "application/json" })
+    public List<BookNoXmlRootElement> getBookCollection2(List<BookNoXmlRootElement> bs) throws Exception {
+        if (bs == null || bs.size() != 2) {
+            throw new RuntimeException();
+        }
+        return bs;
+    }
+    
+    @GET
+    @Path("/collections")
+    @Produces({"application/xml", "application/json" })
+    public List<Book> getBookCollection() throws Exception {
+        return new ArrayList<Book>(books.values());
+    }
+    
     @POST
     @Path("/array")
     @Produces("application/xml")
@@ -233,6 +334,15 @@ public class BookStore {
         return doGetBook(s1 + s2);
     }
     
+    @GET
+    @Path("/segment/matrix-list")
+    public Book getBookByMatrixListParams(@MatrixParam("first") List<String> list) throws Exception {
+        if (list.size() != 2) {
+            throw new RuntimeException();
+        }
+        return doGetBook(list.get(0) + list.get(1));
+    }
+    
     @GET
     @Path("/bookheaders/")
     public Book getBookByHeader(@HeaderParam("BOOK") List<String> ids) throws Exception {
@@ -335,6 +445,48 @@ public class BookStore {
         return doGetBook(id);
     }
     
+    
+    
+    @GET
+    @Path("/books/response/{bookId}/")
+    @Produces("application/xml")
+    public Response getBookAsResponse(@PathParam("bookId") String id) throws BookNotFoundFault {
+        Book entity = doGetBook(id);
+        EntityTag etag = new EntityTag(Integer.toString(entity.hashCode()));
+        return Response.ok().tag(etag).entity(entity).build();
+    }
+    
+    @GET
+    @Path("/books/{bookId}/cglib")
+    @Produces("application/xml")
+    public Book getBookCGLIB(@PathParam("bookId") String id) throws BookNotFoundFault {
+        return createCglibProxy(doGetBook(id));
+    }
+    
+    @GET
+    @Path("/the books/{bookId}/")
+    @Produces("application/xml")
+    public Book getBookWithSpace(@PathParam("bookId") String id) throws BookNotFoundFault {
+        return doGetBook(id);
+    }
+    
+    @GET
+    @Path("/books/search")
+    @Produces("application/xml")
+    public Book getBook(@Context SearchContext searchContext) 
+        throws BookNotFoundFault {
+        
+        SearchCondition<Book> sc = searchContext.getCondition(Book.class);
+        if (sc == null) {
+            throw new BookNotFoundFault("Search exception");
+        }
+        List<Book> found = sc.findAll(books.values());
+        if (found.size() != 1) {
+            throw new BookNotFoundFault("Single book is expected");
+        }
+        return found.get(0);
+    }
+    
     @GET
     @Path("/books/{bookId}/")
     @Produces("text/xml")
@@ -396,19 +548,63 @@ public class BookStore {
                                      doGetBook("123"));
     }
     
+    @POST
+    @Path("/books/element/echo")
+    public JAXBElement<Book> echoBookElement(JAXBElement<Book> element) throws Exception {
+        return element;
+    }
+    
+    @SuppressWarnings("unchecked")
+    @POST
+    @Path("/books/element/echo")
+    public JAXBElement<? super Book> echoBookElementWildcard(JAXBElement<? extends Book> element) 
+        throws Exception {
+        return (JAXBElement<? super Book>)element;
+    }
+    
     @GET
     @Path("/books/adapter")
     @XmlJavaTypeAdapter(BookInfoAdapter.class)
     public BookInfo getBookAdapter() throws Exception {
         return new BookInfo(doGetBook("123"));
     }
+    @POST
+    @Path("/books/adapter-list")
+    @XmlJavaTypeAdapter(BookInfoAdapter.class)
+    @Consumes("application/xml")
+    @Produces({"application/xml", "application/json" })
+    public List<BookInfo> getBookAdapterList(@XmlJavaTypeAdapter(BookInfoAdapter.class) 
+                                              List<BookInfo> collection) 
+        throws Exception {
+        if (collection.size() != 1) {
+            throw new WebApplicationException(400);
+        }
+        return collection;
+    }
     
     @GET
     @Path("/books/interface/adapter")
-    public BookInfoInterface getBookAdapter2() throws Exception {
+    public BookInfoInterface getBookAdapterInterface() throws Exception {
         return new BookInfo2(doGetBook("123"));
     }
     
+    @GET
+    @Path("/books/interface/adapter-list")
+    public List<? extends BookInfoInterface> getBookAdapterInterfaceList() throws Exception {
+        List<BookInfoInterface> list = new ArrayList<BookInfoInterface>();
+        list.add(new BookInfo2(doGetBook("123")));
+        return list;
+    }
+    
+    @GET
+    @Path("/books/adapter-list")
+    @XmlJavaTypeAdapter(BookInfoAdapter.class)
+    public List<? extends BookInfo> getBookAdapterList() throws Exception {
+        List<BookInfo> list = new ArrayList<BookInfo>();
+        list.add(new BookInfo(doGetBook("123")));
+        return list;
+    }
+    
     @PathParam("bookId")
     public void setBookId(String id) {
         currentBookId = id;
@@ -517,7 +713,7 @@ public class BookStore {
     @Produces("application/xml")
     @Consumes("text/xml")
     public Book addBookCustomFailure(Book book, @Context HttpServletResponse response) {
-        response.setStatus(333);
+        response.setStatus(233);
         response.addHeader("CustomHeader", "CustomValue");
         book.setId(888);
         return book;
@@ -684,7 +880,50 @@ public class BookStore {
         c.setCD(cds.values());
         return c;
     }
-    
+
+    @GET
+    @Path("quotedheaders")
+    public Response getQuotedHeader() {
+        return Response.
+                ok().
+                header("SomeHeader1", "\"some text, some more text\"").
+                header("SomeHeader2", "\"some text\"").
+                header("SomeHeader2", "\"quoted,text\"").
+                header("SomeHeader2", "\"even more text\"").
+                header("SomeHeader3", "\"some text, some more text with inlined \\\"\"").
+                header("SomeHeader4", "\"\"").
+                build();
+    }
+
+    @GET
+    @Path("badlyquotedheaders")
+    public Response getBadlyQuotedHeader(@QueryParam("type")int t) {
+        Response.ResponseBuilder rb = Response.ok();
+        switch(t) {
+        case 0:
+            // problem: no trailing quote - doesn't trigger AbstractClient.parseQuotedHeaderValue
+            rb.header("SomeHeader0", "\"some text");
+            break;
+        case 1:
+            // problem: text doesn't end with " - triggers AbstractClient.parseQuotedHeaderValue
+            rb.header("SomeHeader1", "\"some text, some more text with inlined \\\"");
+            break;
+        case 2:
+            // problem: no character after \ - doesn't trigger AbstractClient.parseQuotedHeaderValue
+            rb.header("SomeHeader2", "\"some te\\");
+            break;
+        case 3:
+            // problem: mix of plain text and quoted text in same line - doesn't trigger
+            // AbstractClient.parseQuotedHeaderValue
+            rb.header("SomeHeader3", "some text").header("SomeHeader3", "\"other quoted\", text").
+                header("SomeHeader3", "blah");
+            break;
+        default:
+            throw new RuntimeException("Don't know how to handle type: " + t);
+        }
+        return rb.build();
+    }
+
     @Path("/interface")
     public BookSubresource getBookFromSubresource() {
         return new BookSubresourceImpl();
@@ -706,8 +945,8 @@ public class BookStore {
         cds.put(cd1.getId(), cd1);
     }
     
-    @XmlJavaTypeAdapter(BookInfoAdapter.class)
-    private static interface BookInfoInterface {
+    @XmlJavaTypeAdapter(BookInfoAdapter2.class)
+    static interface BookInfoInterface {
         String getName();
         
         long getId();
@@ -754,6 +993,18 @@ public class BookStore {
             super(b);
         }
     }
+        
+    public static class BookInfoAdapter2 extends XmlAdapter<Book, BookInfo2> {
+        @Override
+        public Book marshal(BookInfo2 v) throws Exception {
+            return new Book(v.getName(), v.getId());
+        }
+
+        @Override
+        public BookInfo2 unmarshal(Book b) throws Exception {
+            return new BookInfo2(b);
+        }
+    }
     
     public static class BookInfoAdapter extends XmlAdapter<Book, BookInfo> {
 
@@ -795,6 +1046,19 @@ public class BookStore {
         } 
         
     }
+    
+    private Book createCglibProxy(final Book book) {
+        final InvocationHandler handler = new InvocationHandler() {
+
+            public Object invoke(Object object, Method method, Object[] args) throws Throwable {
+                return method.invoke(book, args);
+            }
+            
+        };
+        return (Book)ProxyHelper.getProxy(this.getClass().getClassLoader(), 
+                                    new Class[]{Book.class}, 
+                                    handler);
+    }
 }
 
 
