@@ -23,6 +23,7 @@ import java.io.File;
 import java.io.InputStream;
 import java.net.HttpURLConnection;
 import java.net.URL;
+import java.net.URLConnection;
 import java.net.URLEncoder;
 import java.util.ArrayList;
 import java.util.Collection;
@@ -46,6 +47,7 @@ import org.apache.commons.httpclient.methods.PutMethod;
 import org.apache.commons.httpclient.methods.RequestEntity;
 import org.apache.cxf.helpers.IOUtils;
 import org.apache.cxf.io.CachedOutputStream;
+import org.apache.cxf.jaxrs.client.ClientWebApplicationException;
 import org.apache.cxf.jaxrs.client.JAXRSClientFactory;
 import org.apache.cxf.jaxrs.client.JAXRSClientFactoryBean;
 import org.apache.cxf.jaxrs.client.ResponseReader;
@@ -72,14 +74,18 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
     }
     
     @Test
-    public void testMalformedAcceptType() {
-        WebClient wc = 
-            WebClient.create("http://localhost:" + PORT + "/bookstore/books/123");
-        wc.accept("application");
-        Response r = wc.get();
-        assertEquals(406, r.getStatus());
+    public void testProxyWrongAddress() throws Exception {
+        BookStore store = JAXRSClientFactory.create("http://localhost:8080/wrongaddress", 
+                                                    BookStore.class);
+        try {
+            store.getBook("123");
+            fail("ClientWebApplicationException expected");
+        } catch (ClientWebApplicationException ex) {
+            // expected
+        }
     }
     
+    @Test
     public void testGetBookWithCustomHeader() throws Exception {
         
         String endpointAddress =
@@ -534,40 +540,16 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         assertEquals(204, response.getStatus());
     }
     
-    @Test
-    public void testEmptyPut() throws Exception {
-        WebClient wc = 
-            WebClient.create("http://localhost:" 
-                             + PORT + "/bookstore/emptyput");
-        Response response = wc.put(null);
-        assertEquals(204, response.getStatus());
-        
-        response = wc.put("");
-        assertEquals(204, response.getStatus());
-    }
-    
     @Test
     public void testEmptyPostProxy() throws Exception {
-        String address = "http://localhost:" + PORT;
         JAXRSClientFactoryBean bean = new JAXRSClientFactoryBean(); 
-        bean.setAddress(address);
+        bean.setAddress("http://localhost:" + PORT);
         bean.setResourceClass(BookStore.class);
         BookStore store = bean.create(BookStore.class);
         store.emptypost();
         assertEquals(204, WebClient.client(store).getResponse().getStatus());
     }
     
-    @Test
-    public void testEmptyPostProxy2() throws Exception {
-        String address = "http://localhost:" + PORT;
-        JAXRSClientFactoryBean bean = new JAXRSClientFactoryBean();
-        bean.setAddress(address);
-        bean.setResourceClass(BookStore.class);
-        BookStore store = bean.create(BookStore.class);
-        store.emptypostNoPath();
-        assertEquals(204, WebClient.client(store).getResponse().getStatus());
-    }
-    
     @Test
     public void testGetBookByEncodedQuery() throws Exception {
         getAndCompareAsStrings("http://localhost:" + PORT + "/bookstore/bookquery?"
@@ -950,14 +932,6 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
                                "application/xml", 200);
     }
     
-    @Test
-    public void testGetBookByMatrixParametersInTheMiddle() throws Exception {
-        getAndCompareAsStrings(
-            "http://localhost:" + PORT + "/bookstore/segment;first=12;second=3/matrix-middle",
-            "resources/expected_get_book123.txt",
-            "application/xml", 200);
-    }
-    
     @Test
     public void testGetBookByHeader() throws Exception {
         getAndCompareAsStrings("http://localhost:" + PORT + "/bookstore/bookheaders",
@@ -1177,19 +1151,6 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         }
     }
     
-    @Test
-    public void testAddBookEmptyContent() throws Exception {
-        Response r = WebClient.create("http://localhost:" + PORT + "/bookstore/books").post(null);
-        assertEquals(400, r.getStatus());
-    }
-    
-    @Test
-    public void testAddBookEmptyContentWithNullable() throws Exception {
-        Book defaultBook = 
-            WebClient.create("http://localhost:" + PORT + "/bookstore/books/null").post(null, Book.class);
-        assertEquals("Default Book", defaultBook.getName());
-    }
-    
     @Test
     public void testAddBook() throws Exception {
         doAddBook("http://localhost:" + PORT + "/bookstore/books");               
@@ -1247,14 +1208,39 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         try {
             int result = httpclient.executeMethod(put);
             assertEquals(200, result);
-            InputStream expected = getClass().getResourceAsStream("resources/expected_update_book.txt");
-            assertEquals(getStringFromInputStream(expected), 
-                         getStringFromInputStream(put.getResponseBodyAsStream()));
         } finally {
             // Release current connection to the connection pool once you are
             // done
             put.releaseConnection();
         }
+
+        // Verify result
+        endpointAddress = "http://localhost:" + PORT + "/bookstore/books/123";
+        URL url = new URL(endpointAddress);
+        URLConnection connect = url.openConnection();
+        connect.addRequestProperty("Accept", "application/xml");
+        InputStream in = connect.getInputStream();
+        assertNotNull(in);
+
+        InputStream expected = getClass().getResourceAsStream("resources/expected_update_book.txt");
+
+        assertEquals(getStringFromInputStream(expected), getStringFromInputStream(in));
+
+        // Roll back changes:
+        File input1 = new File(getClass().getResource("resources/expected_get_book123.txt").toURI());
+        PutMethod put1 = new PutMethod(endpointAddress);
+        RequestEntity entity1 = new FileRequestEntity(input1, "text/xml; charset=ISO-8859-1");
+        put1.setRequestEntity(entity1);
+        HttpClient httpclient1 = new HttpClient();
+
+        try {
+            int result = httpclient1.executeMethod(put);
+            assertEquals(200, result);
+        } finally {
+            // Release current connection to the connection pool once you are
+            // done
+            put1.releaseConnection();
+        }
     }  
     
     @Test
@@ -1295,14 +1281,39 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         try {
             int result = httpclient.executeMethod(put);
             assertEquals(200, result);
-            InputStream expected = getClass().getResourceAsStream("resources/expected_update_book.txt");
-            assertEquals(getStringFromInputStream(expected), 
-                         getStringFromInputStream(put.getResponseBodyAsStream()));
         } finally {
             // Release current connection to the connection pool once you are
             // done
             put.releaseConnection();
         }
+
+        // Verify result
+        endpointAddress = "http://localhost:" + PORT + "/bookstore/books/123";
+        URL url = new URL(endpointAddress);
+        URLConnection connection = url.openConnection();
+        connection.addRequestProperty("Accept", "application/xml");
+        InputStream in = connection.getInputStream();
+        assertNotNull(in);
+
+        InputStream expected = getClass().getResourceAsStream("resources/expected_update_book.txt");
+
+        assertEquals(getStringFromInputStream(expected), getStringFromInputStream(in));
+
+        // Roll back changes:
+        File input1 = new File(getClass().getResource("resources/expected_get_book123.txt").toURI());
+        PutMethod put1 = new PutMethod(endpointAddress);
+        RequestEntity entity1 = new FileRequestEntity(input1, "text/xml; charset=ISO-8859-1");
+        put1.setRequestEntity(entity1);
+        HttpClient httpclient1 = new HttpClient();
+
+        try {
+            int result = httpclient1.executeMethod(put);
+            assertEquals(200, result);
+        } finally {
+            // Release current connection to the connection pool once you are
+            // done
+            put1.releaseConnection();
+        }
     } 
     
     @Test
@@ -1466,9 +1477,9 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
     public void testUriInfoMatchedResources() throws Exception {
         getAndCompare("http://localhost:" + PORT + "/bookstore/"
                       + "booksubresource/123/chapters/sub/1/matched-resources", 
-                      "[class org.apache.cxf.systest.jaxrs.Chapter, " 
+                      "[class org.apache.cxf.systest.jaxrs.BookStore, " 
                       + "class org.apache.cxf.systest.jaxrs.Book, "
-                      + "class org.apache.cxf.systest.jaxrs.BookStore]", 
+                      + "class org.apache.cxf.systest.jaxrs.Chapter]", 
                       "text/plain", "text/plain", 200);
     }
     
@@ -1476,17 +1487,17 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
     public void testUriInfoMatchedResourcesWithObject() throws Exception {
         getAndCompare("http://localhost:" + PORT + "/bookstore/"
                       + "booksubresource/123/chaptersobject/sub/1/matched-resources", 
-                      "[class org.apache.cxf.systest.jaxrs.Chapter, " 
+                      "[class org.apache.cxf.systest.jaxrs.BookStore, " 
                       + "class org.apache.cxf.systest.jaxrs.Book, "
-                      + "class org.apache.cxf.systest.jaxrs.BookStore]", 
+                      + "class org.apache.cxf.systest.jaxrs.Chapter]", 
                       "text/plain", "text/plain", 200);
     }
 
     @Test
     public void testUriInfoMatchedUrisDecode() throws Exception {
-        String expected = "[/bookstore/booksubresource/123/chapters/sub/1/matched!uris, "
+        String expected = "[/bookstore/booksubresource/123/, "
                           + "/bookstore/booksubresource/123/chapters/sub/1/, "
-                          + "/bookstore/booksubresource/123/]";
+                          + "/bookstore/booksubresource/123/chapters/sub/1/matched!uris]";
         getAndCompare("http://localhost:" + PORT + "/bookstore/"
                       + "booksubresource/123/chapters/sub/1/matched%21uris?decode=true", 
                       expected, "text/plain", "text/plain", 200);
@@ -1495,9 +1506,9 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
     @Test
     public void testUriInfoMatchedUrisNoDecode() throws Exception {
         //note '%21' instead of '!'
-        String expected = "[/bookstore/booksubresource/123/chapters/sub/1/matched%21uris, "
+        String expected = "[/bookstore/booksubresource/123/, "
             + "/bookstore/booksubresource/123/chapters/sub/1/, "
-            + "/bookstore/booksubresource/123/]";
+            + "/bookstore/booksubresource/123/chapters/sub/1/matched%21uris]";
         getAndCompare("http://localhost:" + PORT + "/bookstore/"
                       + "booksubresource/123/chapters/sub/1/matched%21uris?decode=false", 
                       expected,
